==> fotoalbum-0.33.tar.bz2: fotoalbum-0.33/src/shared/SimpleXMLTransfer.h <==
/*
 * Copyright (C) 2003-2007, 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef _SIMPLEXMLTRANSFER_H_
# define _SIMPLEXMLTRANSFER_H_

# include <string>
# include <vector>
# include <fstream>

typedef enum
{
  STATE_IDLE,
    STATE_START,
    STATE_INITTAG,
    STATE_INPUT,
    STATE_READNAME,
    STATE_WAIT_CONT,
    STATE_CONT_START,
    STATE_ENDTAG,
    STATE_WAIT_ATTR_NAME,
    STATE_ATTR_NAME,
    STATE_ATTR_VAL1,
    STATE_ATTR_VAL,
    STATE_ATTR_VAL_START,
    STATE_COMMENT,
    STATE_ATTR_VAL_WAIT_EQ,
    STATE_EXPECT_CLOSE,
    STATE_COMMENT_END1,
    STATE_COMMENT_END2
} T_State;


/** \brief Standard exception for SimpleXMLTransfer.
 * 
 *  This is the standard exception which is thrown by many methods of
 *  SimpleXMLTransfer if an operation fails.
 */
class XMLException
{
public:
  /**
   * Create a new XMLException with a given <code>message</code>.
   */
  XMLException(std::string message);
  
  /**
   * Return a <code>const char*</code> to the exception's message.
   */
  const char* what();
  
private:
  std::string myMessage;
};


class SimpleXMLTransfer;

/** \brief Simple XML parser class.
 * 
 */
class SimpleXMLTransfer
{
public:

  /// @name Construction, Destruction
  //@{
  
  /**
   * Create an empty instance.
   */
  SimpleXMLTransfer();
  
  /**
   * Read from a file.
   * \param source Path to the file.
   * 
   * \todo
   * Wenn sie mit <code>file://</code> beginnt, wird eine lokale Datei gelesen, ansonsten
   * wird ein Netzwerkzugriff versucht.
   * Wenn der Name mit <code>.gz</code> endet, wird angenommen, dass die Datei mit GnuZip
   * gepackt ist.
   */
  SimpleXMLTransfer(std::string source);
  
  /**
   * Read from a stream.
   */
  SimpleXMLTransfer(std::istream& in);
  
  /**
   * Copy
   */
  SimpleXMLTransfer(SimpleXMLTransfer* source);
  
  /**
   * Destructor
   */
  ~SimpleXMLTransfer();
  
  /**
   * Merge with another one
   */
  void merge(SimpleXMLTransfer* b);  
  //@}
  
  /// @name Get content
  //@{
  
  /**
   * Returns the content as a std::string.
   */
  std::string getContentString();
  
  /**
   * Return the content as a double precision floating point value.
   */
  double getContentDouble();
  
  /**
   * Return the content as an integer value.
   * 
   * Possible notation: see @convToInt
   */
  int getContentInt();
  
  //@}
  
  /// @name Misc get
  //@{
  
  /**
   * Return the name of the object.
   */
  std::string getName();
  
  /**
   * Returns a string describing the source of this xml data
   */
  std::string getSourceDescr() { return(sourcedescr); };
  
  //@}
  
  /// @name Get children and related methods
  //@{
  
  /**
   * Return the number of children.
   */
  int getChildCount() const;

  /**
   * Return the index of the first child with the name
   * <code>child</code> or -1 if no such child exists.
   */
  int indexOfChild(std::string child);
  
  /**
   * Return the index of the first child with the name
   * <code>child</code> or -1 if no such child exists.
   * Start searching at <code>nStartIdx</code>.
   */
  int indexOfChild(std::string child, int nStartIdx);
  
  /**
   * Return the child with the index <code>childIndex</code>.
   */
  SimpleXMLTransfer* getChildAt(int childIndex);

  /**
   * Returns the first child named child. Search is started at index idx+1.
   * If such a child was found, idx contains its index.
   */
  SimpleXMLTransfer* getChild(std::string child, int& idx);
  
  /**
   * Returns the first child named <code>child</code> or throws
   * an <code>XMLException</code> if no child with this name exists
   * and <code>fCreate == false</code>. If the latter is <code>true</code>,
   * the child is created.
   * 
   * It's legal to access children's children, e.g.
   * <code>getChild("eat.breakfast.bread")</code>
   * instead of
   * <code>getChild("eat").getChild("breakfast").getChild("bread")</code>
   */
  SimpleXMLTransfer* getChild(std::string child, bool fCreate = false);
        
  //@}
  
  /// @name Get attribute and related methods
  //@{
  
  /** 
   * Returns number of attributes
   */
  int getAttributeCount() { return(attrName.size()); };
  
  /**
   * Returns the index of the first attribute named
   * <code>attr</code> or -1 of none exists.
   */
  int indexOfAttribute(std::string attr) const;

  /**
   * Returns name of attribute at index <code>index</code>.
   */
  std::string attributeName(unsigned int index);

  /**
   * Returns value of attribute at index <code>index</code>.
   */
  std::string attributeVal (unsigned int index);
  
  //@}
  
  /// @name Get attribute val, exception
  //@{
  
  /**
   * Returns the value of the attribute <code>attr</code> as a string or
   * throws an XMLException if the attribute does not exist.
   */
  std::string attribute(std::string attr);
  
  /**
   * Returns the integer value of the attribute <code>attr</code> or
   * throws an XMLException if the attribute does not exist.
   * 
   * Possible notation: see @convToInt
   */
  int attributeAsInt(std::string attr);
  
  /**
   * Returns the double precicion floating point value of the attribute
   * or throws an XMLException if the attribute does not exist.
   */
  double attributeAsDouble(unsigned int index);
  
  /**
   * Returns the floating point value of the attribute <code>attr</code> or
   * throws an XMLException if the attribute does not exist.
   */
  float attributeAsFloat(std::string attr);
  
  /**
   * Returns the double precicion floating point value of the attribute
   * <code>attr</code> or throws an XMLException if the attribute does not exist.
   */
  double attributeAsDouble(std::string attr);
  
  //@}
  
  /// @name Get attribute val, default
  //@{
  
  /**
   * Returns the value of the attribute <code>attr</code> as a std::string.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   */
  std::string attribute(std::string attr,
                        std::string defaultVal);
  
  /**
   * Returns the integer value of the attribute <code>attr</code>.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   * 
   * Possible notation: see @convToInt
   */
  int attributeAsInt(std::string attr,
                     int         defaultVal);
  
  /**
   * Returns the double precision floating point value of the attribute
   * <code>attr</code>.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   */
  double attributeAsDouble(std::string  attr,
                           double       defaultVal);
  
  //@}
  
  /// @name Get attribute deep (exception, default)
  //@{
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   */
  std::string getString(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Be careful: creates attribute with and returns default value if there
   * is no such attribute.
   */
  std::string getString(std::string path, std::string stringDefault);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   * 
   * Possible notation: see @convToInt
   */
  int getInt(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Be careful: creates attribute with and returns default value if there
   * is no such attribute.
   * 
   * Possible notation: see @convToInt
   */
  int getInt(std::string path, int nDefault);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   */
  double getDouble(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Creates attribute with and returns default value if there
   * is no such attribute.
   */
  double getDouble(std::string path, double dDefault);
    
  //@}
  
  /// @name Output
  //@{
  
  /**
   * Print the element's data to a stream.
   * \param out Output stream.
   */
  void print(std::ostream& out, int nIndent=0, int nIndentStep=2, int nLineBreak=80);
  
  /**
   * Print the element's data to a stream (without header).
   * \param out Output stream.
   */
  void printNoHeader(std::ostream& out, int nIndent=0, int nIndentStep=2, int nLineBreak=80);
  
  //@}
  
  /// @name Delete, Remove
  //@{
  
  /**
   * Delete attributes. If an attribute does not exist an exception
   * will be thrown.
   */
  void delAttribute(std::string attribute);
  
  /**
   * Removes the child at position <code>nIndex</code> from the list of
   * children. The child's memory will not be freed!
   */
  void removeChildAt(int nIndex);
  
  /**
   * Removes the child <code>XMLPtr</code> from the list of
   * children. The child's memory will not be freed!
   */
  void removeChild(SimpleXMLTransfer* XMLPtr);
  
  /**
   * Removes comments
   */
  void removeComments();
  
  //@}
  
  /// @name Replace
  //@{
  
  /**
   * Replaces the child XMLPtrOld with XMLPtrNew.
   * The memory of XMLPtrOld will not be freed!
   */
  void replaceChild(SimpleXMLTransfer* XMLPtrOld,
                    SimpleXMLTransfer* XMLPtrNew);
  
  //@}
    
  /// @name Misc set
  //@{
  
  /**
   * Set the content.
   */
  void setContent(std::string newContent);
  
  /**
   * Set the name.
   */
  void setName(std::string name);
  
  //@}

  /// @name Set/add/overwrite attributes
  //@{
  
  /**
   * Add an attribute.
   * There are no checks of any kind!
   */
  void addAttribute(std::string const& attributeName,
                    std::string const& attributeVal);
  
  /**
   * Add an attribute.
   * There are no checks of any kind!
   */
  void addAttribute(std::string const& attributeName,
                    long int    attributeVal);
  
  /**
   * Adds an attribute. Throws XMLException if there already is an attribute 
   * with the same name.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttribute(std::string attribute, std::string value);
  
  /**
   * Adds an attribute. Throws XMLException if there already is an attribute 
   * with the same name.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttribute(std::string attribute, long int nValue);
  
  /**
   * Overwrites value of attribute at <code>index</code>. Throws an exception
   * if there is no attribute at <code>index</code>.
   * 
   * setAttributeOverwrite is more high-level.
   */
  void overwriteAttribute(unsigned int index, std::string val);
     
  /**
   * Adds an attribute. If there already is an attribute
   * with the same name, it will be overwritten.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttributeOverwrite(std::string attribute, const std::string value);
  
  /**
   * Adds an attribute. If there already is an attribute
   * with the same name, it will be overwritten.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttributeOverwrite(std::string attribute, const long int attributeVal);
  
  /**
   * Makes sure that an attribute exists. If it does not, it is created with <code>defaultVal</code>.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void makeSureAttributeExists(std::string attribute, const char* defaultVal);
  
  //@}

  /// @name Add child
  //@{
  
  /**
   * Add a child.
   * There are no checks of any kind!
   */
  void addChild(SimpleXMLTransfer* child);
  
  /**
   * Insert child at position idx
   */
  void insertChild(SimpleXMLTransfer* child, int idx);
  
  //@}

  /// @name Sort
  //@{
    
  /**
   * Sort children by a given attribute. The attributes will
   * be compared as strings.
   */
  void sortChildrenString(std::string attributeName);
  
  /**
   * Sort children by a given attribute. The attributes will
   * be compared as double precision floating point values.
   */
  void sortChildrenDouble(std::string attributeName);
  
  /**
   * Sort children by a given attribute and compare function
   */
  void sortChildren(std::string attributeName, int (*pSortFun)(std::string a, std::string b));
  
  //@}
  
  /// @name Compare
  //@{
    
  /**
   * Recursively compares <code>item</code> with itself. 
   * Returns <code>true</code> if they are equal.
   * The order of children does not matter.
   */
  bool equals(SimpleXMLTransfer* item);
  
  /**
   * Recursively compares <code>item</code> with itself. 
   * Returns <code>true</code> if they are equal.
   * The order of children does matter.
   */
  bool equalsOrdered(SimpleXMLTransfer* item);
  
  //@} 
 
  /**
   * Convert a std::string to make it storable as an
   * attributes' value in an XML file.
   * The conversion does not conform to unicode, but depends 
   * on the system's character set.
   */
  static std::string* convToXML(std::string* in, std::string* strBuf);
    
  /**
   * Possible notation for integer-values: decimal, hexadecimal with 
   * prefixed '0x' or binary with prefixed '0b'.
   */
  static int    convToInt(std::string value);
  
  
private:
  
  /**
   * Name of this element
   */
  std::string                          myName;
  
  /**
   * This elements' children
   */
  std::vector<SimpleXMLTransfer*>      children;
  
  /**
   * This elements' attribute names. Same order as @attrVal.
   */
  std::vector<std::string>             attrName;
  
  /**
   * This elements' attribute values. Same order as @attrName.
   */
  std::vector<std::string>             attrVal;
  
  /**
   * Comments inside of this element. Same order as @commentPos 
   */
  std::vector<std::string>             comment;
  
  /**
   * Positions of comments inside of this element. Same order as @comment
   */
  std::vector<int>                     commentPos;
  
  /**
   * Content of this element
   */
  std::string*                         content;
  
  /**
   * Only for internal purposes!
   */
  void readStream(std::istream& in, char*& acInBuf, int& nBufSize, int& nBufFill, int& nBufIdx);
  
  /**
   * Only for internal purposes!
   */
  SimpleXMLTransfer(std::istream& in, std::string strDescr, char*& acBuf, int& nBufSize, int& nBufFill, int& nBufIdx);
  
  
  /**
   */
  float  convToFloat(std::string value);
  
  double convToDouble(std::string value);
  
  /**
   * Converts a std::string (attribute value in an XML file)
   * into a Unicode-<code>std::string</code>.
   * <br>
   * Does not work in all cases as some things are converted
   * wrong or can cause an XMLException.
   * <br>
   * The representation of <code>in</code> is subjected to some
   * restrictions.
   * Only these entities are converted correctly:
   * <code>&lt;</code>,
   * <code>&gt;</code>,
   * <code>&amp;</code>,
   * <code>&quot;</code>
   * and characters represented as <code>&#xHEXVAL;</code>.
   */
  void convFromXML(char* pBuf, int nLen);
  
  /**
   * 
   */
  static void print(std::ostream& out,
                    int           nIndent,
                    std::string   val);
  
  /**
   * describes the source of the data
   */
  std::string sourcedescr;
};

#endif

==> fotoalbum-0.201.tar.bz2: fotoalbum-0.201/src/shared/SimpleXMLTransfer.h <==
/*
 * Copyright (C) 2003-2007, 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef _SIMPLEXMLTRANSFER_H_
# define _SIMPLEXMLTRANSFER_H_

# include <string>
# include <vector>
# include <fstream>

typedef enum
{
  STATE_IDLE,
    STATE_START,
    STATE_INITTAG,
    STATE_INPUT,
    STATE_READNAME,
    STATE_WAIT_CONT,
    STATE_CONT_START,
    STATE_ENDTAG,
    STATE_WAIT_ATTR_NAME,
    STATE_ATTR_NAME,
    STATE_ATTR_VAL1,
    STATE_ATTR_VAL,
    STATE_ATTR_VAL_START,
    STATE_COMMENT,
    STATE_ATTR_VAL_WAIT_EQ,
    STATE_EXPECT_CLOSE,
    STATE_COMMENT_END1,
    STATE_COMMENT_END2
} T_State;


/** \brief Standard exception for SimpleXMLTransfer.
 * 
 *  This is the standard exception which is thrown by many methods of
 *  SimpleXMLTransfer if an operation fails.
 */
class XMLException
{
public:
  /**
   * Create a new XMLException with a given <code>message</code>.
   */
  XMLException(std::string message);
  
  /**
   * Return a <code>const char*</code> to the exception's message.
   */
  const char* what();
  
private:
  std::string myMessage;
};


class SimpleXMLTransfer;

/** \brief Simple XML parser class.
 * 
 */
class SimpleXMLTransfer
{
public:

  /// @name Construction, Destruction
  //@{
  
  /**
   * Create an empty instance.
   */
  SimpleXMLTransfer();
  
  /**
   * Read from a file.
   * \param source Path to the file.
   * 
   * \todo
   * Wenn sie mit <code>file://</code> beginnt, wird eine lokale Datei gelesen, ansonsten
   * wird ein Netzwerkzugriff versucht.
   * Wenn der Name mit <code>.gz</code> endet, wird angenommen, dass die Datei mit GnuZip
   * gepackt ist.
   */
  SimpleXMLTransfer(std::string source);
  
  /**
   * Read from a stream.
   */
  SimpleXMLTransfer(std::istream& in);
  
  /**
   * Copy
   */
  SimpleXMLTransfer(SimpleXMLTransfer* source);
  
  /**
   * Destructor
   */
  ~SimpleXMLTransfer();
  
  /**
   * Merge with another one
   */
  void merge(SimpleXMLTransfer* b);  
  //@}
  
  /// @name Get content
  //@{
  
  /**
   * Returns the content as a std::string.
   */
  std::string getContentString();
  
  /**
   * Return the content as a double precision floating point value.
   */
  double getContentDouble();
  
  /**
   * Return the content as an integer value.
   * 
   * Possible notation: see @convToInt
   */
  int getContentInt();
  
  //@}
  
  /// @name Misc get
  //@{
  
  /**
   * Return the name of the object.
   */
  std::string getName();
  
  /**
   * Returns a string describing the source of this xml data
   */
  std::string getSourceDescr() { return(sourcedescr); };
  
  //@}
  
  /// @name Get children and related methods
  //@{
  
  /**
   * Return the number of children.
   */
  int getChildCount() const;

  /**
   * Return the index of the first child with the name
   * <code>child</code> or -1 if no such child exists.
   */
  int indexOfChild(std::string child);
  
  /**
   * Return the index of the first child with the name
   * <code>child</code> or -1 if no such child exists.
   * Start searching at <code>nStartIdx</code>.
   */
  int indexOfChild(std::string child, int nStartIdx);
  
  /**
   * Return the child with the index <code>childIndex</code>.
   */
  SimpleXMLTransfer* getChildAt(int childIndex);

  /**
   * Returns the first child named child. Search is started at index idx+1.
   * If such a child was found, idx contains its index.
   */
  SimpleXMLTransfer* getChild(std::string child, int& idx);
  
  /**
   * Returns the first child named <code>child</code> or throws
   * an <code>XMLException</code> if no child with this name exists
   * and <code>fCreate == false</code>. If the latter is <code>true</code>,
   * the child is created.
   * 
   * It's legal to access children's children, e.g.
   * <code>getChild("eat.breakfast.bread")</code>
   * instead of
   * <code>getChild("eat").getChild("breakfast").getChild("bread")</code>
   */
  SimpleXMLTransfer* getChild(std::string child, bool fCreate = false);
        
  //@}
  
  /// @name Get attribute and related methods
  //@{
  
  /** 
   * Returns number of attributes
   */
  int getAttributeCount() const { return(attrName.size()); };
  
  /**
   * Returns the index of the first attribute named
   * <code>attr</code> or -1 of none exists.
   */
  int indexOfAttribute(std::string attr) const;

  /**
   * Returns name of attribute at index <code>index</code>.
   */
  std::string attributeName(unsigned int index);

  /**
   * Returns value of attribute at index <code>index</code>.
   */
  std::string attributeVal (unsigned int index);
  
  //@}
  
  /// @name Get attribute val, exception
  //@{
  
  /**
   * Returns the value of the attribute <code>attr</code> as a string or
   * throws an XMLException if the attribute does not exist.
   */
  std::string attribute(std::string attr);
  
  /**
   * Returns the integer value of the attribute <code>attr</code> or
   * throws an XMLException if the attribute does not exist.
   * 
   * Possible notation: see @convToInt
   */
  int attributeAsInt(std::string attr);
  
  /**
   * Returns the double precicion floating point value of the attribute
   * or throws an XMLException if the attribute does not exist.
   */
  double attributeAsDouble(unsigned int index);
  
  /**
   * Returns the floating point value of the attribute <code>attr</code> or
   * throws an XMLException if the attribute does not exist.
   */
  float attributeAsFloat(std::string attr);
  
  /**
   * Returns the double precicion floating point value of the attribute
   * <code>attr</code> or throws an XMLException if the attribute does not exist.
   */
  double attributeAsDouble(std::string attr);
  
  //@}
  
  /// @name Get attribute val, default
  //@{
  
  /**
   * Returns the value of the attribute <code>attr</code> as a std::string.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   */
  std::string attribute(std::string attr,
                        std::string defaultVal);
  
  /**
   * Returns the integer value of the attribute <code>attr</code>.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   * 
   * Possible notation: see @convToInt
   */
  int attributeAsInt(std::string attr,
                     int         defaultVal);
  
  /**
   * Returns the double precision floating point value of the attribute
   * <code>attr</code>.
   * If the attribute does not exist the function will return the
   * default value <code>defaultVal</code>.
   */
  double attributeAsDouble(std::string  attr,
                           double       defaultVal);
  
  //@}
  
  /// @name Get attribute deep (exception, default)
  //@{
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   */
  std::string getString(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Be careful: creates attribute with and returns default value if there
   * is no such attribute.
   */
  std::string getString(std::string path, std::string stringDefault);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   * 
   * Possible notation: see @convToInt
   */
  int getInt(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Be careful: creates attribute with and returns default value if there
   * is no such attribute.
   * 
   * Possible notation: see @convToInt
   */
  int getInt(std::string path, int nDefault);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Throws exception if the attribute can't be found.
   */
  double getDouble(std::string path);
  
  /**
   * Quick access function to an attribute of the element or of a
   * child's child.
   * Creates attribute with and returns default value if there
   * is no such attribute.
   */
  double getDouble(std::string path, double dDefault);
    
  //@}
  
  /// @name Output
  //@{
      
  /**
   * Print the element's data to a stream.
   * \param out Output stream.
   */
  void print(std::ostream& out, int nIndent=0, int nIndentStep=2, int nLineBreak=80);
  
  /**
   * Print the element's data to a stream (without header).
   * \param out Output stream.
   */
  void printNoHeader(std::ostream& out, int nIndent=0, int nIndentStep=2, int nLineBreak=80);
  
  /**
   * returns non zero on error
   */
  int SaveToFile(std::string strFilename, int nIndent=0, int nIndentStep=2, int nLineBreak=80);
  //@}
  
  /// @name Delete, Remove
  //@{
  
  /**
   * Delete attributes. If an attribute does not exist an exception
   * will be thrown.
   */
  void delAttribute(std::string attribute);
  
  /**
   * Removes the child at position <code>nIndex</code> from the list of
   * children. The child's memory will not be freed!
   */
  void removeChildAt(int nIndex);
  
  /**
   * Removes the child <code>XMLPtr</code> from the list of
   * children. The child's memory will not be freed!
   */
  void removeChild(SimpleXMLTransfer* XMLPtr);
  
  /**
   * Removes comments
   */
  void removeComments();
  
  //@}
  
  /// @name Replace
  //@{
  
  /**
   * Replaces the child XMLPtrOld with XMLPtrNew.
   * The memory of XMLPtrOld will not be freed!
   */
  void replaceChild(SimpleXMLTransfer* XMLPtrOld,
                    SimpleXMLTransfer* XMLPtrNew);
  
  //@}
    
  /// @name Misc set
  //@{
  
  /**
   * Set the content.
   */
  void setContent(std::string newContent);
  
  /**
   * Set the name.
   */
  void setName(std::string name);
  
  //@}

  /// @name Set/add/overwrite attributes
  //@{
  
  /**
   * Add an attribute.
   * There are no checks of any kind!
   */
  void addAttribute(std::string const& attributeName,
                    std::string const& attributeVal);
  
  /**
   * Add an attribute.
   * There are no checks of any kind!
   */
  void addAttribute(std::string const& attributeName,
                    long int    attributeVal);
  
  /**
   * Adds an attribute. Throws XMLException if there already is an attribute 
   * with the same name.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttribute(std::string attribute, std::string value);
  
  /**
   * Adds an attribute. Throws XMLException if there already is an attribute 
   * with the same name.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttribute(std::string attribute, long int nValue);
  
  /**
   * Overwrites value of attribute at <code>index</code>. Throws an exception
   * if there is no attribute at <code>index</code>.
   * 
   * setAttributeOverwrite is more high-level.
   */
  void overwriteAttribute(unsigned int index, std::string val);
     
  /**
   * Adds an attribute. If there already is an attribute
   * with the same name, it will be overwritten.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttributeOverwrite(std::string attribute, const std::string value);
  
  /**
   * Adds an attribute. If there already is an attribute
   * with the same name, it will be overwritten.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void setAttributeOverwrite(std::string attribute, const long int attributeVal);
  
  /**
   * Makes sure that an attribute exists. If it does not, it is created with <code>defaultVal</code>.
   * <code>attribute</code> can be something like <code>pictures.holiday.malibu</code>
   * The last item of the path is taken as the name of the attribute. Non-existing elements
   * in the path are automatically created.
   */
  void makeSureAttributeExists(std::string attribute, const char* defaultVal);
  
  //@}

  /// @name Add child
  //@{
  
  /**
   * Add a child.
   * There are no checks of any kind!
   */
  void addChild(SimpleXMLTransfer* child);
  
  /**
   * Insert child at position idx
   */
  void insertChild(SimpleXMLTransfer* child, int idx);
  
  //@}

  /// @name Sort
  //@{
    
  /**
   * Sort children by a given attribute. The attributes will
   * be compared as strings.
   */
  void sortChildrenString(std::string attributeName);
  
  /**
   * Sort children by a given attribute. The attributes will
   * be compared as double precision floating point values.
   */
  void sortChildrenDouble(std::string attributeName);
  
  /**
   * Sort children by a given attribute and compare function
   */
  void sortChildren(std::string attributeName, int (*pSortFun)(std::string a, std::string b));
  
  //@}
  
  /// @name Compare
  //@{
    
  /**
   * Recursively compares <code>item</code> with itself. 
   * Returns <code>true</code> if they are equal.
   * The order of children does not matter.
   */
  bool equals(SimpleXMLTransfer* item);
  
  /**
   * Recursively compares <code>item</code> with itself. 
   * Returns <code>true</code> if they are equal.
   * The order of children does matter.
   */
  bool equalsOrdered(SimpleXMLTransfer* item);
  
  //@} 
 
  /**
   * Convert a std::string to make it storable as an
   * attributes' value in an XML file.
   * The conversion does not conform to unicode, but depends 
   * on the system's character set.
   */
  static std::string* convToXML(std::string* in, std::string* strBuf);
    
  /**
   * Possible notation for integer-values: decimal, hexadecimal with 
   * prefixed '0x' or binary with prefixed '0b'.
   */
  int    convToInt(std::string value);
  
  /**
   */
  float  convToFloat(std::string value);
  
  double convToDouble(std::string value);
  
private:
  
  /**
   * Name of this element
   */
  std::string                          myName;
  
  /**
   * This elements' children
   */
  std::vector<SimpleXMLTransfer*>      children;
  
  /**
   * This elements' attribute names. Same order as @attrVal.
   */
  std::vector<std::string>             attrName;
  
  /**
   * This elements' attribute values. Same order as @attrName.
   */
  std::vector<std::string>             attrVal;
  
  /**
   * Comments inside of this element. Same order as @commentPos 
   */
  std::vector<std::string>             comment;
  
  /**
   * Positions of comments inside of this element. Same order as @comment
   */
  std::vector<int>                     commentPos;
  
  /**
   * Content of this element
   */
  std::string*                         content;
  
  /**
   * Only for internal purposes!
   */
  void readStream(std::istream& in, char*& acInBuf, int& nBufSize, int& nBufFill, int& nBufIdx);
  
  /**
   * Only for internal purposes!
   */
  SimpleXMLTransfer(std::istream& in, std::string strDescr, char*& acBuf, int& nBufSize, int& nBufFill, int& nBufIdx);
      
  /**
   * Converts a std::string (attribute value in an XML file)
   * into a Unicode-<code>std::string</code>.
   * <br>
   * Does not work in all cases as some things are converted
   * wrong or can cause an XMLException.
   * <br>
   * The representation of <code>in</code> is subjected to some
   * restrictions.
   * Only these entities are converted correctly:
   * <code>&lt;</code>,
   * <code>&gt;</code>,
   * <code>&amp;</code>,
   * <code>&quot;</code>
   * and characters represented as <code>&#xHEXVAL;</code>.
   */
  void convFromXML(char* pBuf, int nLen);
  
  /**
   * 
   */
  static void print(std::ostream& out,
                    int           nIndent,
                    std::string   val);
  
  /**
   * describes the source of the data
   */
  std::string sourcedescr;
};

#endif
